from libcpp cimport bool as c_bool
from libcpp.string cimport string as c_string

from escape.core.array cimport double_array_t
from escape.core.data cimport  data_t, datastack_t
from escape.core.kernel cimport  kernel_t
from escape.core.setting cimport string_setting_t
from escape.core.model cimport model_t
from escape.core.parameter cimport parameter_t

cdef extern from "escape/core.h" namespace "escape::core":

    cdef cppclass model_t:
        model_t()
        model_t(model_t   ) except+
        void call "operator()" (c_bool) except+
        double cost(c_bool, c_bool) except+
        double last_cost() except+
        void set_data(data_t) except+
        data_t data() except +
        double_array_t simulation() except +
        double_array_t residuals() except +
        int dof() except +
        size_t data_length() except +
        void shake() except +
        
        c_string name() except +
        void set_name(c_string   ) except +
        parameter_t parameter(size_t) except +
        size_t num_of_params() except +
        
    cdef cppclass modelstack_t:

        modelstack_t()
        modelstack_t(modelstack_t) except+

        void call "operator()" (c_bool) except+
        double cost "cost" (c_bool, c_bool) except+

        int dof() except+

        void add(model_t) except+
        void set(size_t idx, model_t) except+
        
        void set_data(datastack_t stack) except+

        size_t size() except+
        void shake() except +
        
        void erase(size_t idx) except+
        void erase_all() except+

        int data_length() except +
        model_t model(size_t) except +

        c_string name() except +
        void set_name(c_string) except +
        parameter_t parameter(size_t) except +
        size_t num_of_params() except +

cdef extern from "escape/core.h" namespace "escape":
    model_t cpp_create_model "escape::model"(
        c_string, kernel_t, data_t, string_setting_t, string_setting_t) except +
    modelstack_t cpp_create_modelstack "escape::modelstack"(c_string) except +
    modelstack_t cpp_create_modelstack2 "escape::modelstack"(c_string, model_t) except +

        
cdef class model_obj:
    cdef model_t * _ptr
    cdef set_ptr(self, model_t * ptr)
    @staticmethod
    cdef model_obj from_ptr(model_t *_ptr)

cdef class modelstack_obj:

    cdef modelstack_t * _ptr
    cdef set_ptr(self, modelstack_t * ptr)
    
    @staticmethod
    cdef modelstack_obj from_ptr(modelstack_t *_ptr)
