from libcpp cimport bool as c_bool
from libcpp.string cimport string as c_string


from escape.core.array cimport double_array_t
from escape.core.parameter cimport parameter_t
from escape.core.setting cimport string_setting_t, int_setting_t
from escape.scattering.material cimport material_t

cdef extern from "escape/scattering.h" namespace "escape::scattering":

	cdef cppclass roughness_t:
		roughness_t()
		roughness_t(roughness_t   ) except +
		double rms() except +
		double lateral() except +
		double hurst() except +
		
		c_string name() except +
		void set_name(c_string   ) except +
		parameter_t parameter(size_t) except +
		size_t num_of_params() except +
		
		
	cdef cppclass layer_t:
		layer_t()
		layer_t(layer_t   ) except +
		
		material_t material  () except +
		roughness_t roughness  () except +        
		double thkn() except +
		
		c_string name() except +
		void set_name(c_string   ) except +
		parameter_t parameter(size_t) except +
		size_t num_of_params() except +
		

	cdef cppclass layerstack_t:
		layerstack_t()
		layerstack_t(layerstack_t   ) except +
		void add(layer_t   lay) except +
		void insert(size_t idx, layer_t   lay) except +
		void set(size_t idx, layer_t   lay) except +
		void erase(size_t idx) except +
		void erase_all() except +
		size_t size() except +
		
		c_string name() except +
		void set_name(c_string   ) except +
		parameter_t parameter(size_t) except +
		size_t num_of_params() except +
		

	cdef cppclass multilayer_t:
		multilayer_t()
		multilayer_t(multilayer_t   ) except +
		void add_layer "add" (layer_t   lay) except +
		void add_stack "add" (layerstack_t   stack) except +
		void set_layer "set" (size_t idx, layer_t   lay) except +
		void set_stack "set" (size_t idx, layerstack_t   lay) except +
		
		void  move_up(size_t idx) except+
		void  move_down(size_t idx) except+
		
		void insert_layer "insert" (size_t idx, layer_t   lay) except +
		void insert_stack "insert" (size_t idx, layerstack_t   lay) except +
		void erase(size_t idx) except +
		void erase_all() except +
		size_t size() except +
		
		double_array_t profile_sld0() except +
		
		c_string name() except +
		void set_name(c_string   ) except +
		parameter_t parameter(size_t) except +
		size_t num_of_params() except +
		
	
	
	roughness_t cpp_roughness "escape::roughness"(
		c_string, parameter_t, parameter_t, parameter_t, string_setting_t )
	
	roughness_t cpp_roughness_short "escape::roughness"(c_string, parameter_t)
	
	layer_t cpp_layer "escape::layer" (
		c_string, material_t, parameter_t, roughness_t) except+
		
	layer_t cpp_magnetic_layer "escape::layer" (
		c_string, material_t, parameter_t, parameter_t, parameter_t, roughness_t) except+
	
	layer_t cpp_substrate "escape::substrate" (
		c_string, material_t, roughness_t) except+
		
	layer_t cpp_magnetic_substrate "escape::substrate" (
		c_string, material_t, parameter_t, parameter_t, roughness_t) except+
		
	layer_t cpp_airenv "escape::air_environment" (c_string) except+
	layer_t cpp_environment "escape::environment" (
		c_string, material_t, roughness_t) except+
	
	layerstack_t cpp_layerstack "escape::layer_stack" (
		c_string, int_setting_t) except+
	layerstack_t cpp_layerstack_layer "escape::layer_stack" (layer_t) except+
	
	multilayer_t cpp_multilayer "escape::multilayer" (
		c_string, layer_t, layer_t)


cdef class roughness_obj:
	cdef roughness_t * _ptr
	cdef set_ptr(self, roughness_t * ptr)
	@staticmethod
	cdef roughness_obj from_ptr(roughness_t *_ptr)


cdef class layer_obj:
	cdef layer_t * _ptr
	cdef set_ptr(self, layer_t * ptr)
	@staticmethod
	cdef layer_obj from_ptr(layer_t *_ptr)
    
    
cdef class layerstack_obj:
	cdef layerstack_t * _ptr
	cdef set_ptr(self, layerstack_t * ptr)
	@staticmethod
	cdef layerstack_obj from_ptr(layerstack_t *_ptr)


cdef class multilayer_obj:
	cdef multilayer_t * _ptr
	cdef set_ptr(self, multilayer_t * ptr)
	@staticmethod
	cdef multilayer_obj from_ptr(multilayer_t *_ptr)
    
    
    
