from libcpp cimport bool as c_bool
from libcpp.string cimport string as c_string

from escape.core.array cimport double_array_t
from escape.core.parameter cimport parameter_t
from escape.core.objects cimport variable_t

from escape.core.functor cimport functor_t
from escape.core.setting cimport int_setting_t, double_setting_t, string_setting_t


cdef extern from "escape/scattering.h" namespace "escape::scattering":

    cdef cppclass unitcell_t:
        unitcell_t()
        unitcell_t(unitcell_t) except +

        c_string name() except +
        void set_name(c_string) except +
        parameter_t parameter(size_t) except +
        size_t num_of_params() except +

    cdef cppclass material_t:
        material_t()
        material_t(material_t) except +

        size_t numslices() except +
        material_t at(size_t) except +

        c_string name() except +
        void set_name(c_string) except +
        parameter_t parameter(size_t) except +
        size_t num_of_params() except +
        
    material_t cpp_generic_material "escape::generic_material"(
		c_string, parameter_t, parameter_t, parameter_t, parameter_t, 
		double_setting_t, parameter_t, parameter_t, parameter_t )
    
    material_t cpp_generic_magnetic_material "escape::generic_material"(
        c_string, parameter_t, parameter_t, parameter_t, parameter_t, parameter_t, 
        double_setting_t, parameter_t, parameter_t, parameter_t )
    
    material_t cpp_generic_material_f "escape::generic_material"(
        c_string, functor_t, functor_t, functor_t,
        functor_t, double_setting_t,
        functor_t,
        functor_t,
        functor_t)
    
    material_t cpp_generic_magnetic_material_f "escape::generic_material"(
        c_string, functor_t, functor_t, functor_t,
        functor_t, functor_t, double_setting_t,
        functor_t,
        functor_t,
        functor_t)
    
    material_t cpp_generic_gradient_material "escape::generic_material"(
		c_string, functor_t, functor_t, functor_t,
		functor_t, double_setting_t,
        functor_t,
        functor_t,
        functor_t, int_setting_t)
    
    material_t cpp_generic_magnetic_gradient_material "escape::generic_material"(
        c_string, functor_t, functor_t, functor_t, functor_t,
        functor_t, double_setting_t,
        functor_t,
        functor_t,
        functor_t, int_setting_t)
    
    material_t cpp_amorphous_material "escape::amorphous_material"(
        c_string, parameter_t, parameter_t)
    
    material_t cpp_amorphous_material_f "escape::amorphous_material"(
        c_string, functor_t, functor_t)
    
    material_t cpp_amorphous_magnetic_material "escape::amorphous_material"(
        c_string, parameter_t, parameter_t, parameter_t)
    
    material_t cpp_amorphous_magnetic_material_f "escape::amorphous_material"(
        c_string, functor_t, functor_t, functor_t)
        
    material_t cpp_amorphous_gradient_material "escape::amorphous_material"(
        c_string, variable_t, functor_t, functor_t, int_setting_t)
    
    material_t cpp_amorphous_gradient_magnetic_material "escape::amorphous_material"(
        c_string, variable_t, functor_t, functor_t, functor_t, int_setting_t)
    
    material_t cpp_crystal_material "escape::crystal_material"(
        c_string, parameter_t, parameter_t, parameter_t, parameter_t, 
        double_setting_t, unitcell_t )
    
    material_t cpp_crystal_magnetic_material "escape::crystal_material"(
        c_string, parameter_t, parameter_t, parameter_t, parameter_t, parameter_t, 
        double_setting_t, unitcell_t )
    
    material_t cpp_crystal_gradient_material "escape::crystal_material"(
        c_string, functor_t, 
        functor_t, functor_t, functor_t, 
        double_setting_t, unitcell_t, int_setting_t numslices )
    
    material_t cpp_crystal_material_f "escape::crystal_material"(
        c_string, functor_t, 
        functor_t, functor_t, functor_t, 
        double_setting_t, unitcell_t)
    
    material_t cpp_crystal_gradient_magnetic_material "escape::crystal_material"(
        c_string, functor_t, 
        functor_t, functor_t, functor_t, functor_t, 
        double_setting_t, unitcell_t, int_setting_t numslices )
    
    material_t cpp_crystal_magnetic_material_f "escape::crystal_material"(
        c_string, functor_t, 
        functor_t, functor_t, functor_t, functor_t, 
        double_setting_t, unitcell_t)
    
    unitcell_t cpp_unitcell "escape::unitcell"(
		c_string, parameter_t, parameter_t, parameter_t, parameter_t, 
		parameter_t, parameter_t)
    
    unitcell_t cpp_gradient_unitcell "escape::unitcell"(
        c_string, functor_t, functor_t, functor_t,
        functor_t, 
        functor_t, functor_t)
    
    unitcell_t cpp_strained_unitcell "escape::strained_unitcell"(
        c_string, parameter_t, unitcell_t refucell)
    
    unitcell_t cpp_strained_gradient_unitcell "escape::strained_unitcell"(
        c_string, functor_t, unitcell_t refucell)
    

cdef class unitcell_obj:
    cdef unitcell_t * _ptr
    cdef set_ptr(self, unitcell_t * ptr)

    @staticmethod
    cdef unitcell_obj from_ptr(unitcell_t * _ptr)

cdef class material_obj:
    cdef material_t * _ptr
    cdef set_ptr(self, material_t * ptr)
    @staticmethod
    cdef material_obj from_ptr(material_t * _ptr)

