from libcpp cimport bool as c_bool
from libcpp.vector cimport vector
from libcpp.string cimport string as c_string

from escape.core.functor cimport cplx_functor_t
from escape.core.variable cimport variable_t
from escape.core.parameter cimport parameter_t

from escape.scattering.material cimport material_t
from escape.scattering.geometry cimport geometry_t, vertex_t

cdef extern from "escape/scattering.h" namespace "escape::scattering":
    cdef cppclass potential_t:
    
        potential_t()
        potential_t(potential_t) except +
        
        potential_t operator+(potential_t) except +
        potential_t operator-(potential_t) except +
        
        double complex sld0(double, double, double) except +
        cplx_functor_t fsld0(variable_t, variable_t, variable_t) except +
        
        c_string to_x3d() except +
        c_string name() except +
        void set_name(c_string) except +
        parameter_t parameter(size_t) except +
        size_t num_of_params() except +

ctypedef vector[vertex_t] vertex_vector_t

cdef extern from "escape/scattering.h" namespace "escape":
    
    potential_t cpp_potential_shape3d "escape::potential"(
        c_string, variable_t, variable_t, variable_t,
        material_t, geometry_t,
        parameter_t , parameter_t , parameter_t ,
        vertex_vector_t) 
    
    potential_t cpp_potential_box "escape::box"(
        c_string, variable_t, variable_t, variable_t,
        material_t, parameter_t , parameter_t , parameter_t ,
        parameter_t, parameter_t, parameter_t,
        vertex_vector_t) 
    
    potential_t cpp_potential_pyramid "escape::pyramid"(
        c_string, variable_t, variable_t, variable_t,
        material_t,
        parameter_t, parameter_t,
        parameter_t, parameter_t,
        vertex_t center_bot, vertex_t center_top,
        parameter_t, parameter_t, parameter_t,
        vertex_vector_t) 
    
    
    potential_t cpp_potential_sphere "escape::sphere"(
        c_string, variable_t, variable_t, variable_t,
        material_t, parameter_t , parameter_t,
        vertex_vector_t) 
    
    potential_t cpp_potential_cylinder "escape::cylinder"(
        c_string, variable_t, variable_t, variable_t,
        material_t, parameter_t , parameter_t,
        parameter_t, parameter_t,
        vertex_vector_t) 

cdef class potential_obj:
    cdef potential_t * _ptr
    cdef set_ptr(self, potential_t * ptr)
    @staticmethod
    cdef potential_obj from_ptr(potential_t *_ptr)
    
    
